@@ -1,3 +1,9 @@
+# modified on June 17, 2021
+#  (1) drop glmmADMB since it is not in mainstream repositories
+
+# modified on June 13, 2021
+#  (1) added glmmADMB
+#
 # modified on Feb. 15, 2021
 #  (1) rename 'nSubj' back to 'n' and 'nCellPerSubj' back to 'm' to be consistent with other R functions in this package
 #  (2) use asymptotic distribution of test statistics to calculate power to get stable results and to improve speed. 
@@ -196,28 +202,28 @@ powerEQTL.scRNAseq.sim.default = function(
     pvalVec = unlist(mclapply(1:nSim, function(i) {
       # simulate data under alternative hypothesis: slope not equal to zero
       simDati = simDat.eQTL.scRNAseq(nSubj=nSubj,
-        nCellPerSubj = nCellPerSubj,
-        # probability that an excess zero occurs
-        zero.p = zero.p,
-        m.int = m.int,
-        # standard deviation of the random intercept
-        sigma.int = sigma.int,
-        # slope for genotype in NB distribution: mu = exp(beta0 + beta1*SNP)
-        slope = slope,
-        # dispersion parameter of NB distribution
-        # the smaller theta is, the larger variance of NB random variable is
-        theta  = theta,
-        MAF = MAF # SNP MAF
+                                     nCellPerSubj = nCellPerSubj,
+                                     # probability that an excess zero occurs
+                                     zero.p = zero.p,
+                                     m.int = m.int,
+                                     # standard deviation of the random intercept
+                                     sigma.int = sigma.int,
+                                     # slope for genotype in NB distribution: mu = exp(beta0 + beta1*SNP)
+                                     slope = slope,
+                                     # dispersion parameter of NB distribution
+                                     # the smaller theta is, the larger variance of NB random variable is
+                                     theta  = theta,
+                                     MAF = MAF # SNP MAF
       )
-
-     
+      
+      
       #########################
       # test if slope is significantly different from zero 
       res.try = try(f <- glmmTMB::glmmTMB(formula = counts ~ geno + (1|id), 
-      	      data = simDati,
-              ziformula = ~ 1, 
-	      family = nbinom2), silent = TRUE)
-
+                                          data = simDati,
+                                          ziformula = ~ 1, 
+                                          family = nbinom2), silent = TRUE)
+      
       aa = attr(res.try, which="class")
       if(aa == "try-error")
       {
@@ -225,10 +231,13 @@ powerEQTL.scRNAseq.sim.default = function(
       } else {
         pval = summary(f)$coefficients$cond[2, 4]
       }
-
+      
       return(pval)
-
+      
     }, mc.cores = nCores))
+    
+  } else {
+    stop("Currently available choices for 'estMethod' are GLMMadaptive and glmmTMB!")
   }
 
   # power
@@ -336,44 +345,44 @@ powerEQTL.scRNAseq.sim.default.exact = function(
     res.sim = mclapply(1:nSim, function(i) {
       # simulate data under alternative hypothesis: slope not equal to zero
       simDati = simDat.eQTL.scRNAseq(nSubj=nSubj,
-        nCellPerSubj = nCellPerSubj,
-        # probability that an excess zero occurs
-        zero.p = zero.p,
-        m.int = m.int,
-        # standard deviation of the random intercept
-        sigma.int = sigma.int,
-        # slope for genotype in NB distribution: mu = exp(beta0 + beta1*SNP)
-        slope = slope,
-        # dispersion parameter of NB distribution
-        # the smaller theta is, the larger variance of NB random variable is
-        theta  = theta,
-        MAF = MAF # SNP MAF
+                                     nCellPerSubj = nCellPerSubj,
+                                     # probability that an excess zero occurs
+                                     zero.p = zero.p,
+                                     m.int = m.int,
+                                     # standard deviation of the random intercept
+                                     sigma.int = sigma.int,
+                                     # slope for genotype in NB distribution: mu = exp(beta0 + beta1*SNP)
+                                     slope = slope,
+                                     # dispersion parameter of NB distribution
+                                     # the smaller theta is, the larger variance of NB random variable is
+                                     theta  = theta,
+                                     MAF = MAF # SNP MAF
       )
-
+      
       # simulate data under null hypothesis: slope = 0
       simDat0 = simDat.eQTL.scRNAseq(nSubj=nSubj,
-        nCellPerSubj = nCellPerSubj,
-        # probability that an excess zero occurs
-        zero.p = zero.p,
-        m.int = m.int,
-        # standard deviation of the random intercept
-        sigma.int = sigma.int,
-        # slope for genotype in NB distribution: mu = exp(beta0 + beta1*SNP)
-        slope = 0,
-        # dispersion parameter of NB distribution
-        # the smaller theta is, the larger variance of NB random variable is
-        theta  = theta,
-        MAF = MAF # SNP MAF
+                                     nCellPerSubj = nCellPerSubj,
+                                     # probability that an excess zero occurs
+                                     zero.p = zero.p,
+                                     m.int = m.int,
+                                     # standard deviation of the random intercept
+                                     sigma.int = sigma.int,
+                                     # slope for genotype in NB distribution: mu = exp(beta0 + beta1*SNP)
+                                     slope = 0,
+                                     # dispersion parameter of NB distribution
+                                     # the smaller theta is, the larger variance of NB random variable is
+                                     theta  = theta,
+                                     MAF = MAF # SNP MAF
       )
- 
+      
       
       #########################
       # test if slope is significantly different from zero 
       res.try = try(f <- glmmTMB::glmmTMB(formula = counts ~ geno + (1|id), 
-      	      data = simDati,
-              ziformula = ~ 1, 
-	      family = nbinom2), silent = TRUE)
-
+                                          data = simDati,
+                                          ziformula = ~ 1, 
+                                          family = nbinom2), silent = TRUE)
+      
       aa = attr(res.try, which="class")
       if(aa == "try-error")
       {
@@ -381,12 +390,12 @@ powerEQTL.scRNAseq.sim.default.exact = function(
       } else {
         stat1 = summary(f)$coefficients$cond[2, 3]
       }
-
+      
       res.try0 = try(f0 <- glmmTMB::glmmTMB(formula = counts ~ geno + (1|id), 
-      	      data = simDat0,
-              ziformula = ~ 1, 
-	      family = nbinom2), silent = TRUE)
-
+                                            data = simDat0,
+                                            ziformula = ~ 1, 
+                                            family = nbinom2), silent = TRUE)
+      
       aa0 = attr(res.try0, which="class")
       if(aa0 == "try-error")
       {
@@ -394,14 +403,16 @@ powerEQTL.scRNAseq.sim.default.exact = function(
       } else {
         stat0 = summary(f0)$coefficients$cond[2, 3]
       }
-
+      
       res = c(stat1, stat0)
       return(res)
-
-
+      
+      
     }, mc.cores = nCores)
+  } else {
+    stop("Currently available choices for 'estMethod' are GLMMadaptive and glmmTMB!")
   }
-
+  
   mat = t(sapply(res.sim, function(x) {x}))
   colnames(mat) = c("Ha", "H0")
 
